#!/usr/local/cpanel/3rdparty/bin/perl
# cpanel - bin/jail_safe_crontab.pl               Copyright(c) 2013 cPanel, Inc.
#                                                           All Rights Reserved.
# copyright@cpanel.net                                         http://cpanel.net
# This code is subject to the cPanel license. Unauthorized copying is prohibited

# no for production;
# use warnings;
use strict;
use Cpanel::LoadFile     ();
use Cpanel::TempFile     ();
use Cpanel::PwCache      ();
use Cpanel::Wrap         ();
use Cpanel::Wrap::Config ();
use Cpanel::FindBin      ();

my ( $username, $shell ) = ( Cpanel::PwCache::getpwuid($>) )[ 0, 8 ];

# Not jailed, just pass the request
if ( -x '/usr/bin/crontab' && $shell !~ m/\/(?:no|jail)shell/ ) {
    exec '/usr/bin/crontab', @ARGV;
    print "Failed to execute: /usr/bin/crontab: $!\n";
    exit 1;
}

my ( $crontab_fetch_result, $crontab_text ) = _get_crontab_text();
if ( !$crontab_fetch_result ) {
    print $crontab_text . "\n";
    exit(1);
}

if ( grep { $_ eq '-e' } @ARGV ) {
    _edit_crontab();
}
elsif ( grep { $_ eq '-l' } @ARGV ) {
    _display_crontab();
}
elsif ( grep { $_ eq '-r' } @ARGV ) {
    _remove_crontab();
}
elsif ( grep { $_ =~ m/^-[^ler]/ } @ARGV ) {
    print "Invalid option: $ARGV[0]\n";
    _usage();
}
elsif ( length $ARGV[0] ) {
    _install_crontab_from_file();
}
else {
    _install_crontab_from_stdin();
}

sub _remove_crontab {
    my ( $install_status, $install_message ) = _install_new_crontab('');

    print $install_message . "\n";
    exit(0);
}

sub _display_crontab {
    print $crontab_text;
    exit(0);
}

sub _install_crontab_from_file {
    my ($filename) = @ARGV;

    if ( !-e $filename ) {
        print "'$filename' does not exist.\n";
        _usage();
    }

    my $new_crontab_text = Cpanel::LoadFile::loadfile($filename);

    if ( !length $new_crontab_text ) {
        print "File '$filename' is empty.\n";
        _usage();
    }

    my ( $install_status, $install_message ) = _install_new_crontab($new_crontab_text);

    print $install_message . "\n";
}

sub _install_crontab_from_stdin {
    local $/;

    my $new_crontab_text = readline( \*STDIN );

    if ( !length $new_crontab_text ) {
        print "No input on stdin.\n";
        _usage();
    }

    my ( $install_status, $install_message ) = _install_new_crontab($new_crontab_text);

    print $install_message . "\n";

}

sub _edit_crontab {
    my $temp_obj = Cpanel::TempFile->new();
    my ( $temp_file, $temp_fh ) = $temp_obj->file();
    if ( !$temp_fh ) {
        print "Failed to create a temporary file.\n";
        exit(1);
    }

    print {$temp_fh} $crontab_text;
    close($temp_fh);

    my ( $get_editor_bin_status, $editor_bin ) = _get_editor_bin();
    if ( !$get_editor_bin_status ) {
        print $editor_bin . "\n";
        exit(1);
    }

    system $editor_bin, $temp_file;

    my $new_crontab_text = Cpanel::LoadFile::loadfile($temp_file);

    if ( $new_crontab_text eq $crontab_text ) {
        print "$0: no changes\n";
        exit(0);
    }

    my ( $install_status, $install_message ) = _install_new_crontab($new_crontab_text);

    print $install_message . "\n";
    exit(0);
}

sub _install_new_crontab {
    my ($crontab_text) = @_;

    my $result = Cpanel::Wrap::send_cpwrapd_request(
        'namespace' => 'Cpanel',
        'module'    => 'cron',
        'function'  => 'SAVE',
        'data'      => { 'crontab' => $crontab_text },
        'env'       => Cpanel::Wrap::Config::safe_hashref_of_allowed_env(),
    );

    if ( !$result->{'status'} ) {
        return ( 0, "Cpanel::Wrap::send_cpwrapd_request failed to save crontab: " . $result->{'statusmsg'} );
    }

    return ( 1, "$0: installing new crontab" );

}

sub _get_editor_bin {
    my $editor_bin;
    foreach my $editor ( $ENV{'VISUAL'}, $ENV{'EDITOR'}, 'nano', 'vim', 'pico', 'emacs' ) {
        $editor_bin = Cpanel::FindBin::findbin($editor);
        last if $editor_bin;
    }
    return ( 1, $editor_bin ) if defined $editor_bin;
    return ( 0, "No editor found, please set the VISUAL or EDITOR enviroment variables." );
}

sub _get_crontab_text {

    my $result = Cpanel::Wrap::send_cpwrapd_request(
        'namespace' => 'Cpanel',
        'module'    => 'cron',
        'function'  => 'FETCH',
        'env'       => Cpanel::Wrap::Config::safe_hashref_of_allowed_env(),
    );

    if ( !$result->{'status'} ) {

        return ( 0, "Cpanel::Wrap::send_cpwrapd_request failed to fetch crontab: " . $result->{'statusmsg'} );
    }

    if ( $result->{'data'}->{'denied'} ) {
        return ( 0, "You ($username) are not allowed to use this program ($0)\nSee crontab(1) for more information" );
    }

    if ( !exists $result->{'data'}->{'crontab'} ) {
        return ( 0, "Could not fetch crontab: " . $result->{'statusmsg'} );
    }

    return ( 1, $result->{'data'}->{'crontab'} );
}

sub _usage {
    print <<EOM;
usage:  crontab file
  crontab [ -e | -l | -r ]
  -e  (edit user's crontab)
  -l  (list user's crontab)
  -r  (delete user's crontab)
EOM
    exit(1);
}
